@@ -16,6 +16,7 @@ class WhamResult(coretools.Result):
                  logW: np.ndarray,
                  logZ: np.ndarray,
                  nit: int,
+                 nfev: int,
                  eps: float):
         super().__init__()
         self.logW = logW
@@ -24,6 +25,8 @@ class WhamResult(coretools.Result):
         """`numpy.ndarray` containing the logarithm of the partition function of each state."""
         self.nit = nit
         """The number of performed iterations."""
+        self.nfev = nfev
+        """The number of function evalutations (might differ from nit when using method='minimize')."""
         self.eps = eps
         """The final error in the iterative solution."""
 
@@ -36,7 +39,10 @@ def wham(bias,
         threshold: float = 1e-40,
         verbose: bool = False,
         logZ: Optional[np.ndarray] = None,
-        logW: Optional[np.ndarray] = None):
+        logW: Optional[np.ndarray] = None,
+        normalize: bool = True,
+        method: str = "substitute",
+        minimize_opt: Optional[dict] = None):
     """Compute weights according to binless WHAM.
 
        The main input for this calculation is in the 2D array `bias`.
@@ -48,6 +54,14 @@ def wham(bias,
        It is crucial however to compute the potential according to each of the employed
        Hamiltonian on **all the frames**, not only on those simulated using that Hamiltonian.
 
+       Notice that, unless one passes `normalize=False`, the returned weights are normalized.
+       In most practical cases, this means that one should decide a priori on which Hamiltonian
+       weights should be computed and subtract it first. E.g., to obtain the weights corresponding
+       to the first Hamiltonian, one should likely replace `bias` with `bias-bias[:,0][:,np.newaxis]`.
+       Not doing so, unless bias fluctuations are very small, will result in numerical issues.
+       If using `normalize=False`, weights are **not** normalized and biases are pre-shifted so as
+       to decrease numerical issues.
+
        Combining trajectories of different length
        ------------------------------------------
 
@@ -167,6 +181,17 @@ def wham(bias,
            the bias.  Providing an initial guess that is close to the converged value can speed up
            significantly the convergence. *If logW is provided, logZ is ignored*.
 
+       normalize: bool, optional
+           If False, do not normalize resulting weights. Useful when biases fluctuate a lot and one
+           does not want to choose first on which of the Hamiltonians they should be normalized.
+
+       method: str, optional
+           If "substitute", solve self-consistent equations by substitution. This is the default.
+           If "minimize", use a minimization as in J Chem Phys 136, 144102 (2012).
+
+       minimize_opt: dict, optional
+           If method=="minimize", this dict can be used to pass options to scipy.minimize.
+           Notice that by default the minimization is performed using 'L-BFGS-B'.
     """
 
     # allow tuples or lists
@@ -186,14 +211,22 @@ def wham(bias,
     assert len(traj_weight) == ntraj
     assert len(frame_weight) == nframes
 
+    # divide by T once for all
+    shifted_bias = bias/T
+    # track shifts
+    shifts0 = np.min(shifted_bias, axis=0)
+    shifted_bias -= shifts0[np.newaxis,:]
+    shifts1 = np.min(shifted_bias, axis=1)
+    shifted_bias -= shifts1[:,np.newaxis]
+
     # do exponentials only once
-    expv = np.exp((-bias+np.min(bias,axis=0)[np.newaxis,:])/T)
+    expv = np.exp(-shifted_bias)
 
     if logW is not None:
-        Z = np.matmul(np.exp(logW), expv)
+        Z = np.matmul(np.exp(logW-shifts1), expv)
         Z /= np.sum(Z*traj_weight)
     elif logZ is not None:
-        Z = np.exp(logZ)
+        Z = np.exp(logZ+shifts0)
     else:
         Z = np.ones(ntraj)
 
@@ -201,23 +234,71 @@ def wham(bias,
 
     if verbose:
         sys.stderr.write("WHAM: start\n")
-    for nit in range(maxiter):
-        # find unnormalized weights
+    if method == "substitute":
+        for nit in range(maxiter):
+            # find unnormalized weights
+            weight = 1.0/np.matmul(expv, traj_weight/Z)*frame_weight
+            # update partition functions
+            Z = np.matmul(weight, expv)
+            # normalize the partition functions
+            Z /= np.sum(Z*traj_weight)
+            # monitor change in partition functions
+            eps = np.sum(np.log(Z/Zold)**2)
+            Zold = Z
+            if verbose:
+                sys.stderr.write("WHAM: iteration "+str(nit)+" eps "+str(eps)+"\n")
+            if eps < threshold:
+                break
+        nfev=nit
+    elif method == "minimize":
+        from scipy.optimize import minimize
+        def func(x):
+            x-=np.average(x)
+            Zm1=np.exp(-x)
+            tmp=expv*(traj_weight*Zm1)[np.newaxis,:]
+            tmp1=np.sum(tmp,axis=1)
+            C=np.sum(frame_weight*np.log(tmp1))+np.sum(traj_weight*x)
+            tmp/=tmp1[:,np.newaxis]
+            grad=-np.matmul(frame_weight,tmp)+traj_weight
+            grad-=np.average(grad)
+            return C,grad
+        if minimize_opt is not None:
+            if not "method" in minimize_opt:
+                minimize_opt["method"]="L-BFGS-B"
+            if "jac" in minimize_opt:
+                if not minimize_opt["jac"]:
+                    raise ValueError("minimize_opt['jac'] must be True")
+            else:
+                minimize_opt["jac"]=True
+        else:
+            minimize_opt={}
+            minimize_opt["method"]="L-BFGS-B"
+            minimize_opt["jac"]=True
+        x=np.log(Z)
+        res = minimize(func, x, **minimize_opt)
+        Z=np.exp(res.x)
+        Z/=np.sum(Z*traj_weight)
         weight = 1.0/np.matmul(expv, traj_weight/Z)*frame_weight
-        # normalized them
-        weight /= np.sum(weight)
-        # update partition functions
+        Zold=Z.copy()
         Z = np.matmul(weight, expv)
-        # normalize the partition functions
         Z /= np.sum(Z*traj_weight)
-        # monitor change in partition functions
-        eps = np.sum(np.log(Z/Zold)**2)
-        Zold = Z
-        if verbose:
-            sys.stderr.write("WHAM: iteration "+str(nit)+" eps "+str(eps)+"\n")
-        if eps < threshold:
-            break
+        nit=res.nit
+        nfev=res.nfev
+        eps=np.sum(np.log(Z/Zold)**2)
+    else:
+        raise ValueError("method should be 'minimize' or 'substitute'")
+
+    if normalize:
+        weight *= np.exp((shifts1-np.max(shifts1)))
+        # normalized weights
+        weight /= np.sum(weight)
+        with np.errstate(divide = 'ignore'):
+            logW = np.log(weight)
+    else:
+        logW = np.log(weight) + shifts1
+
     if verbose:
         sys.stderr.write("WHAM: end")
 
-    return WhamResult(logW=np.log(weight), logZ=np.log(Z), nit=nit, eps=eps)
+
+    return WhamResult(logW=logW, logZ=np.log(Z)-shifts0, nit=nit, nfev=nfev, eps=eps)
